#define LED (* (volatile unsigned char*) 0x1C100)

#define UART_TXDATA (* (volatile unsigned char*) 0x1C200)
#define UART_RXDATA (* (volatile unsigned char*) 0x1C204)
#define UART_STATUS (* (volatile unsigned char*) 0x1C208)
#define UART_PRESCALE (* (volatile unsigned short*) 0x1C20C)

#define LINK_TXDATA (* (volatile unsigned char*) 0x1C260)
#define LINK_RXDATA (* (volatile unsigned char*) 0x1C264)
#define LINK_STATUS (* (volatile unsigned char*) 0x1C268)

void _main_int(void)
{
  static unsigned short int count;
  volatile unsigned char ireg;

  ireg = *(volatile unsigned char*)0x18021;
  // Blink with 5 Hz
  if (count>= 9)
  {
    LED = 0;
    count = 0;
  }
  else if (count==4)
  {
    LED = 1;
  }
  count++;
  *(volatile unsigned char*)0x18021 = 0xFF; // Clear all ints
}

void _main(void)
{
  unsigned char c;

  UART_PRESCALE = (40000000 / 115200 / 8 + 1) / 2 - 1; // 115200 Baud
  // UART_PRESCALE = (40000000 / 9600 / 8 + 1) / 2 - 1; // 115200 Baud
  UART_STATUS &= 0xDF; // 8 data bits

  while (1)
  {
    if (LINK_STATUS & 0x01)
    {
      // LINK to QL is ready for a new byte
      if (!(UART_STATUS & 0x02))
      {
        // Byte was received from SER
        c = UART_RXDATA;
        // Send byte to QL
        LINK_TXDATA = c;
      }
    }
    else if (!(UART_STATUS & 0x02))
    {
      // LINK to QL has no room in FIFO, but data arrived from SER
      // Indicate overrun by enabling frame interrupt with blinking LED
      asm volatile ("and.w #0xF8FF,%sr");
    }    
    if (UART_STATUS & 0x01)
    {
      // UART is ready for a new byte
      if (!(LINK_STATUS & 0x02))
      {
        // Byte was received from QL
        c = LINK_RXDATA;
        // Send byte to SER
        UART_TXDATA = c;
      }
    }
  }
}
